@@ -1,5 +1,5 @@
 /*
-  $Id: MbtAPI.h 9077 2011-03-23 10:11:02Z sloot $
+  $Id: MbtAPI.h 13158 2011-09-13 15:28:25Z sloot $
   $URL: https://ilk.uvt.nl/svn/trunk/sources/Mbt3/include/mbt/MbtAPI.h $
 
   Copyright (c) 1998 - 2011
@@ -39,11 +39,14 @@ using namespace Tagger;
 
 class MbtAPI {
  public:
+  static bool GenerateTagger( TimblOpts& );
   static bool GenerateTagger( int, char** );
+  static bool GenerateTagger( const std::string& );
   static bool RunTagger( int, char** );
   MbtAPI( const std::string& );
   MbtAPI( const std::string&, LogStream& );
   ~MbtAPI();
+  bool isInit() const;
   std::string Tag( const std::string& );
  private:
   TaggerClass *tagger;
