@@ -1,5 +1,5 @@
 /*
-  $Id: Tagger.h 9077 2011-03-23 10:11:02Z sloot $
+  $Id: Tagger.h 13736 2011-12-20 16:51:18Z sloot $
   $URL: https://ilk.uvt.nl/svn/trunk/sources/Mbt3/include/mbt/Tagger.h $
 
   Copyright (c) 1998 - 2011
@@ -35,6 +35,10 @@
 #include "timbl/TimblAPI.h"
 
 namespace Tagger {
+
+  std::string Version();
+  std::string VersionName();
+
   const int EMPTY_PATH = -1000000;
 
   class n_best_tuple {
@@ -86,7 +90,7 @@ namespace Tagger {
     void CreateSettingsFile();
     bool set_default_filenames();
     void parse_create_args( Timbl::TimblOpts& Opts );
-    void parse_run_args( Timbl::TimblOpts& Opts );
+    void parse_run_args( Timbl::TimblOpts& Opts, bool = false );
     bool isClone() const { return cloned; };
     void ShowCats( std::ostream& os, std::vector<int>& Pat, int slots );
     bool setLog( LogStream& );
@@ -95,12 +99,14 @@ namespace Tagger {
     void read_listfile( const std::string&, StringHash * );
     static TaggerClass *StartTagger( Timbl::TimblOpts&, LogStream* = 0 );
     static int CreateTagger( Timbl::TimblOpts& );
+    bool isInit() const { return initialized; };
   private:
     LogStream *cur_log;
     sentence mySentence;
     Timbl::TimblAPI *KnownTree;
     Timbl::TimblAPI *unKnownTree;
     std::string Timbl_Options;
+    std::string commonstr;
     std::string knownstr;
     std::string unknownstr;
     std::string uwf;
@@ -124,21 +130,23 @@ namespace Tagger {
     bool dumpflag;
     bool distance_flag;
     bool distrib_flag;
+    bool confidence_flag;
     bool klistflag;
     int Beam_Size;
     std::vector<double> distance_array;
     std::vector<std::string> distribution_array;
+    std::vector<double> confidence_array;
 
     int makedataset( std::istream& infile, bool do_known );
     bool readsettings( std::string& fname );
-    void create_lexicons( const std::string& filename );
+    void create_lexicons();
     int ProcessFile( std::istream&, std::ostream& );
     void ProcessTags( TagInfo * );
     std::string Tag();
     void InitTest( MatchAction );
     bool NextBest( int, int );
     const Timbl::TargetValue *Classify( MatchAction, const std::string&, 
-					const Timbl::ValueDistribution *distribution, 
+					const Timbl::ValueDistribution **distribution, 
 					double& );
     std::string get_result();
     void statistics( int& no_known,
@@ -148,7 +156,7 @@ namespace Tagger {
     std::string pat_to_string( MatchAction, int );
 
     std::string TimblOptStr;
-    int FilterTreshold;
+    int FilterThreshold;
     int Npax;
     int TopNumber;
     bool DoSort;
@@ -176,6 +184,7 @@ namespace Tagger {
     std::string TopNFileName;
     std::string NpaxFileName;
     std::string TestFileName;
+    std::string TestFilePath;
     std::string OutputFileName;
     std::string SettingsFileName;
     std::string SettingsFilePath;
