@@ -1,5 +1,5 @@
 /*
-  $Id: TagLex.cxx 9077 2011-03-23 10:11:02Z sloot $
+  $Id: TagLex.cxx 13736 2011-12-20 16:51:18Z sloot $
   $URL: https://ilk.uvt.nl/svn/trunk/sources/Mbt3/src/TagLex.cxx $
 
   Copyright (c) 1998 - 2011
@@ -62,11 +62,11 @@ namespace Tagger {
     }
   }
   
-  void TagInfo::Prune( int Treshold ){
+  void TagInfo::Prune( int Threshold ){
     map<string,int>::iterator it = TagFreqs.begin();
     while ( it != TagFreqs.end() ){
       double perc = ( (double)it->second * 100 ) / ( double)WordFreq;
-      if ( perc < Treshold )
+      if ( perc < Threshold )
 	TagFreqs.erase( it++ );
       else
 	++it;
