@@ -1,5 +1,5 @@
 /*
-  $Id: Tagger.cxx 9077 2011-03-23 10:11:02Z sloot $
+  $Id: Tagger.cxx 13736 2011-12-20 16:51:18Z sloot $
   $URL: https://ilk.uvt.nl/svn/trunk/sources/Mbt3/src/Tagger.cxx $
 
   Copyright (c) 1998 - 2011
@@ -62,6 +62,9 @@ namespace Tagger {
   using namespace Hash;
   using namespace Timbl;
 
+  string Version() { return VERSION; }
+  string VersionName() { return PACKAGE_STRING; }
+
   const string UNKSTR   = "UNKNOWN";
   
   class BeamData;
@@ -73,7 +76,7 @@ namespace Tagger {
     KnownTree = NULL;
     unKnownTree = NULL;
     TimblOptStr = "+vS -FColumns K: -a IGTREE +D U: -a IB1 ";
-    FilterTreshold = 5;
+    FilterThreshold = 5;
     Npax = 5;
     TopNumber = 100;
     DoSort = false;
@@ -93,6 +96,7 @@ namespace Tagger {
     TopNFileName = "";
     NpaxFileName = "";
     TestFileName = "";
+    TestFilePath = "";
     OutputFileName = "";
     SettingsFileName = "";
     SettingsFilePath = "";
@@ -116,15 +120,17 @@ namespace Tagger {
     dumpflag = false;
     distance_flag = false;
     distrib_flag = false;
+    confidence_flag = false;
     klistflag= false;
     cloned = false;
   }
 
   TaggerClass::TaggerClass( const TaggerClass& in ){
+    cur_log = in.cur_log;
     KnownTree = in.KnownTree;
     unKnownTree = in.unKnownTree;
     TimblOptStr = in.TimblOptStr;
-    FilterTreshold = in.FilterTreshold;
+    FilterThreshold = in.FilterThreshold;
     Npax = in.Npax;
     TopNumber = in.TopNumber;
     DoSort = in.DoSort;
@@ -147,6 +153,7 @@ namespace Tagger {
     TopNFileName = in.TopNFileName;
     NpaxFileName = in.NpaxFileName;
     TestFileName = in.TestFileName;
+    TestFilePath = in.TestFilePath;
     OutputFileName = in.OutputFileName;
     SettingsFileName = in.SettingsFileName;
     SettingsFilePath = in.SettingsFilePath;
@@ -169,11 +176,14 @@ namespace Tagger {
     dumpflag = in.dumpflag;
     distance_flag = in.distance_flag;
     distrib_flag = in.distrib_flag;
+    confidence_flag = in.confidence_flag;
     klistflag = in.klistflag;
     cloned = true;
   }
 
   bool TaggerClass::setLog( LogStream& os ){
+    if ( !cloned )
+      delete cur_log;
     cur_log = new LogStream( os, "mbt-" );
     return true;
   }
@@ -521,12 +531,12 @@ namespace Tagger {
   }
   
 
-  void splits( const string& opts, string& known, string& unknown,
-	       string& kwf, string& uwf ){
+  void splits( const string& opts, string& common,
+	       string& known, string& unknown ){
     xDBG << "splits, opts = " << opts << endl;
     known = "";
     unknown = "";
-    string common = " -FColumns ";
+    common = " -FColumns ";
     bool done_u = false, done_k = false;
     string::size_type k_pos = opts.find( "K:" );
     string::size_type u_pos = opts.find( "U:" );
@@ -571,12 +581,34 @@ namespace Tagger {
     xDBG << "resultaat splits, common = " << common << endl;
     xDBG << "resultaat splits, K = " << known << endl;
     xDBG << "resultaat splits, U = " << unknown << endl;
-    known += common;
-    unknown += common;
-    get_weightsfile_name( known, kwf );
-    get_weightsfile_name( unknown, uwf );
   }
+
+  //**** stuff to process commandline options *********************************
+  // used to convert relative paths to absolute paths
+
+  /**
+   * If you do 'mbt -s some-path/xxx.settingsfile' Timbl can not find the 
+   * tree files.
+   *
+   * Because the settings file can contain relative paths for files these 
+   * paths are converted to absolute paths. 
+   * The relative paths are taken relative to the pos ition of the settings
+   * file, so the path of the settings file is prefixed to the realtive path.
+   *
+   * Paths that do not begin with '/' and do not have as second character ':'
+   *      (C: or X: in windows cygwin) are considered to be relative paths
+   */
   
+  void prefixWithAbsolutePath( string& fileName, 
+			       const string& prefix ) {
+    //    cout << fileName << endl;
+    if ( ( fileName[0] != '/' && fileName[1] != ':' )
+	 && !( fileName[0]== '.' && fileName[1] == '/' ) ){
+      fileName = prefix + fileName;
+    }
+    //    cout << fileName << endl;
+  }
+    
   bool TaggerClass::set_default_filenames( ){
     //
     // and use them to setup the defaults...
@@ -600,28 +632,33 @@ namespace Tagger {
     }
     char affix[32];
     LexFileName = TestFileName;
+    prefixWithAbsolutePath( LexFileName, SettingsFilePath );
     LexFileName += ".lex";
-    if ( FilterTreshold < 10 )
-      sprintf( affix, ".0%1i",  FilterTreshold );
+    if ( FilterThreshold < 10 )
+      sprintf( affix, ".0%1i",  FilterThreshold );
     else
-      sprintf( affix, ".%2i",  FilterTreshold );
+      sprintf( affix, ".%2i",  FilterThreshold );
     if( !knownoutfileflag ){
       K_option_name = TestFileName;
+      prefixWithAbsolutePath( K_option_name, SettingsFilePath );
       K_option_name += ".known.inst.";
       K_option_name += KtmplStr;
     }
     if ( !knowntreeflag ){
       KnownTreeName = TestFileName;
+      prefixWithAbsolutePath( KnownTreeName, SettingsFilePath );
       KnownTreeName += ".known.";
       KnownTreeName += KtmplStr;
     }
     if( !unknownoutfileflag ){
       U_option_name = TestFileName;
+      prefixWithAbsolutePath( U_option_name, SettingsFilePath );
       U_option_name += ".unknown.inst.";
       U_option_name += UtmplStr;
     }
     if ( !unknowntreeflag ){
       UnknownTreeName = TestFileName;
+      prefixWithAbsolutePath( UnknownTreeName, SettingsFilePath );
       UnknownTreeName += ".unknown.";
       UnknownTreeName += UtmplStr;
     }
@@ -630,6 +667,7 @@ namespace Tagger {
     }
     else {
       MTLexFileName = TestFileName;
+      prefixWithAbsolutePath( MTLexFileName, SettingsFilePath );
       MTLexFileName += ".lex.ambi";
       MTLexFileName +=  affix;
     }
@@ -638,14 +676,16 @@ namespace Tagger {
     else {
       sprintf( affix, ".top%d",  TopNumber );
       TopNFileName = TestFileName + affix;
+      prefixWithAbsolutePath( TopNFileName, SettingsFilePath );
     }
     sprintf( affix, ".%dpaxes",  Npax );
     NpaxFileName = TestFileName + affix;
+    prefixWithAbsolutePath( NpaxFileName, SettingsFilePath );
     return true;
   }
     
   void TaggerClass::ProcessTags( TagInfo *TI ){
-    TI->Prune( FilterTreshold );
+    TI->Prune( FilterThreshold );
     TI->CreateStringRepr();
   }
   
@@ -660,11 +700,12 @@ namespace Tagger {
     return false;
   }
   
-  void TaggerClass::create_lexicons( const string& filename ){
+  void TaggerClass::create_lexicons(){
     TagLex TaggedLexicon;
     ifstream lex_file;
     ofstream out_file;
     string Buffer;
+    string filename = TestFilePath + TestFileName;
     if ( filename != "" ){
       if ( ( lex_file.open( filename.c_str(), ios::in ),
 	     !lex_file.good() ) ){
@@ -696,7 +737,7 @@ namespace Tagger {
       out_file.close();
     }
     else {
-      cerr << "couldn't open lexiconfile " << LexFileName << endl;
+      cerr << "couldn't create lexiconfile " << LexFileName << endl;
       exit(EXIT_FAILURE);
     }
     for ( int i=0; i < LexSize; i++ )
@@ -869,7 +910,13 @@ namespace Tagger {
       Timbl_Options[pos] = '+';
       pos = Timbl_Options.find( "-vS", pos+1 );
     }
-    splits( Timbl_Options, knownstr, unknownstr, kwf, uwf );
+    splits( Timbl_Options, commonstr, knownstr, unknownstr );
+    if ( confidence_flag ){
+      if ( commonstr.find("-G") == string::npos ){
+	cerr << "-vcf is specified, but -G is missing in the common Timbl Options" << endl;
+	return false;
+      }
+    }
     if( !knowntreeflag ){
       cerr << "<knowntreefile> not specified" << endl;
       return false;
@@ -878,10 +925,10 @@ namespace Tagger {
       cerr << "<unknowntreefile> not specified" << endl;
       return false;
     }
-    KnownTree = new TimblAPI( knownstr );
+    KnownTree = new TimblAPI( knownstr + commonstr );
     if ( !KnownTree->Valid() )
       return false;
-    unKnownTree = new TimblAPI( unknownstr );
+    unKnownTree = new TimblAPI( unknownstr + commonstr );
     if ( !unKnownTree->Valid() )
       return false;
     // read a previously stored InstanceBase for known words
@@ -894,6 +941,8 @@ namespace Tagger {
       return false;
     }
     else {
+      get_weightsfile_name( knownstr, kwf );
+      get_weightsfile_name( unknownstr, uwf );
       if ( !kwf.empty() ){
 	if ( !KnownTree->GetWeights( kwf ) ){
 	  cerr << "Couldn't read known weights from " << kwf << endl;
@@ -957,13 +1006,14 @@ namespace Tagger {
 	os = &cout;
       ifstream infile;    
       if ( !piped_input ){
-	infile.open(TestFileName.c_str(), ios::in);
+	string inname = TestFilePath + TestFileName;
+	infile.open(inname.c_str(), ios::in);
 	if( infile.bad( )){
-	  cerr << "Cannot read from " << TestFileName << endl;
+	  cerr << "Cannot read from " << inname << endl;
 	  result = 0;
 	}
 	else {
-	  cerr << "Processing data from the file " << TestFileName 
+	  cerr << "Processing data from the file " << inname 
 	       << ":" <<  endl;
 	  result = ProcessFile(infile, *os );
 	}
@@ -990,7 +1040,7 @@ namespace Tagger {
 
   const TargetValue *TaggerClass::Classify( MatchAction Action, 
 					    const string& teststring, 
-					    const ValueDistribution *distribution,
+					    const ValueDistribution **distribution,
 					    double& distance ){
     const TargetValue *answer = 0;
 #if defined(HAVE_PTHREAD)
@@ -998,10 +1048,10 @@ namespace Tagger {
       pthread_mutex_lock( &timbl_lock );
 #endif
     if ( Action == Known ){
-      answer = KnownTree->Classify( teststring, distribution, distance );
+      answer = KnownTree->Classify( teststring, *distribution, distance );
     }
     else {
-      answer = unKnownTree->Classify( teststring, distribution, distance );
+      answer = unKnownTree->Classify( teststring, *distribution, distance );
     }
 #if defined(HAVE_PTHREAD)
     if ( cloned )
@@ -1019,13 +1069,18 @@ namespace Tagger {
     string teststring = pat_to_string( Action, 0 );
     const ValueDistribution *distribution = 0;
     double distance;
-    const TargetValue *answer = Classify( Action, teststring, distribution, distance );
+    const TargetValue *answer = Classify( Action, teststring, &distribution, distance );
     distance_array.resize( mySentence.size() );
     distribution_array.resize( mySentence.size() );
+    confidence_array.resize( mySentence.size() );
     if ( distance_flag )
       distance_array[0] = distance;
-    if ( distrib_flag )
-      distribution_array[0] = distribution->DistToString();
+    if ( distribution ){
+      if ( distrib_flag )
+	distribution_array[0] = distribution->DistToString();
+      if ( confidence_flag )
+	confidence_array[0] = distribution->Confidence( answer );
+    }
     if ( IsActive( DBG ) ){
       LOG << "BeamData::InitPaths( " << mySentence << endl; 
       LOG << " , " << answer << " , " << distribution << " )" << endl;
@@ -1054,12 +1109,16 @@ namespace Tagger {
       const ValueDistribution *distribution = 0;
       double distance;
       const TargetValue *answer = Classify( Action, teststring, 
-					    distribution, distance );
+					    &distribution, distance );
       if ( beam_cnt == 0 ){
 	if ( distance_flag )
 	  distance_array[i_word] = distance;
-	if ( distrib_flag )
-	  distribution_array[i_word] = distribution->DistToString();
+	if ( distribution ){
+	  if ( distrib_flag )
+	    distribution_array[i_word] = distribution->DistToString();
+	  if ( confidence_flag )
+	    confidence_array[i_word] = distribution->Confidence( answer );
+	}
       }
       Beam->NextPath( TheLex, answer, distribution, beam_cnt ); 
       if ( IsActive( DBG ) )
@@ -1145,22 +1204,30 @@ namespace Tagger {
       if ( input_kind == ENRICHED ){
 	result = result + mySentence.getenr(Wcnt) + "\t" 
 	  + mySentence.gettag(Wcnt) + "\t" + tagstring;
-	if ( distance_flag )
-	  result += " " + toString( distance_array[Wcnt] );
+	if ( confidence_flag )
+	  result += " [" + toString( confidence_array[Wcnt] ) + "]";
 	if ( distrib_flag )
 	  result += " " + distribution_array[Wcnt];
+	if ( distance_flag )
+	  result += " " + toString( distance_array[Wcnt] );
 	result += "\n";
       }
       else if ( input_kind == TAGGED ){
 	result = result + mySentence.gettag(Wcnt) + "\t" + tagstring;
-	if ( distance_flag )
-	  result += " " + toString( distance_array[Wcnt] );
+	if ( confidence_flag )
+	  result += " [" + toString( confidence_array[Wcnt] ) + "]";
 	if ( distrib_flag )
 	  result += " " + distribution_array[Wcnt];
+	if ( distance_flag )
+	  result += " " + toString( distance_array[Wcnt] );
 	result += "\n";
       }
-      else
-	result = result + tagstring + " ";
+      else {
+	result = result + tagstring;
+	if ( confidence_flag )
+	  result += "/" + toString( confidence_array[Wcnt] );
+	result += " ";
+      }
     } // end of output loop through one sentence
     if ( input_kind != ENRICHED )
       result = result + mySentence.Eos();
@@ -1262,33 +1329,6 @@ namespace Tagger {
   }
   
   
-//**** stuff to process commandline options *********************************
-
-  // used to convert relative paths to absolute paths
-
-  /**
-   * If you do 'mbt -s some-path/xxx.settingsfile' Timbl can not find the 
-   * tree files.
-   *
-   * Because the settings file can contain relative paths for files these 
-   * paths are converted to absolute paths. 
-   * The relative paths are taken relative to the pos ition of the settings
-   * file, so the path of the settings file is prefixed to the realtive path.
-   *
-   * Paths that do not begin with '/' and do not have as second character ':'
-   *      (C: or X: in windows cygwin) are considered to be relative paths
-   */
-  
-  void prefixWithAbsolutePath( string& fileName, 
-			       const string& prefix ) {
-    //    cout << fileName << endl;
-    if ( ( fileName[0] != '/' && fileName[1] != ':' )
-	 && !( fileName[0]== '.' && fileName[1] == '/' ) ){
-      fileName = prefix + fileName;
-    }
-    //    cout << fileName << endl;
-  }
-  
   bool TaggerClass::readsettings( string& fname ){
     ifstream setfile( fname.c_str(), ios::in);
     if(setfile.bad()){
@@ -1398,9 +1438,14 @@ namespace Tagger {
     return true;
   }
 
-  void TaggerClass::parse_run_args( TimblOpts& Opts ){
+  void TaggerClass::parse_run_args( TimblOpts& Opts, bool as_server ){
     string value;
     bool mood;
+    if ( Opts.Find( 'V', value, mood ) ||
+	 Opts.Find( "version", value, mood ) ){
+      // we already identified ourself. just bail out.
+      exit(EXIT_SUCCESS);
+    }
     if ( Opts.Find( 's', value, mood ) ){
       // if a settingsfile option has been given, read that first
       // and then override with commandline options 
@@ -1518,6 +1563,8 @@ namespace Tagger {
 	  distance_flag = true;
 	if ( opts[i] == "db" )
 	  distrib_flag = true;
+	if ( opts[i] == "cf" )
+	  confidence_flag = true;
       }
       Opts.Delete( 'v' );
     };
@@ -1526,9 +1573,24 @@ namespace Tagger {
 	   << "bailing out, sorry " << endl;
       exit(EXIT_FAILURE);
     }
+    if ( !as_server &&
+	 (!knowntreeflag || !unknowntreeflag) ){
+      cerr << "missing required options. See 'mbt -h' " << endl;
+      exit(EXIT_FAILURE);
+    }
   }
   
   TaggerClass *TaggerClass::StartTagger( TimblOpts& Opts, LogStream* os ){
+    // present yourself to the user
+    //
+    cerr << "mbt " << VERSION << " (c) ILK and CLiPS 1998 - 2011." << endl
+	 << "Memory Based Tagger " << endl
+	 << "Tilburg University" << endl
+	 << "CLiPS Computational Linguistics Group, University of Antwerp"
+	 << endl
+	 << "Based on " << Timbl::VersionName() 
+	 << endl << endl;
+    
     TaggerClass *tagger = new TaggerClass;
     tagger->parse_run_args( Opts );
     if ( os )
@@ -1547,12 +1609,14 @@ namespace Tagger {
     // name for both output files (concatenation of datafile 
     // and pattern string)
     //
-    create_lexicons( TestFileName );
+    create_lexicons();
     if ( TimblOptStr.empty() )
       Timbl_Options = "-a IB1";
     else
       Timbl_Options = TimblOptStr;
-    splits( Timbl_Options, knownstr, unknownstr, kwf, uwf );
+    splits( Timbl_Options, commonstr, knownstr, unknownstr );
+    get_weightsfile_name( knownstr, kwf );
+    get_weightsfile_name( unknownstr, uwf );
     // the testpattern is of the form given in Ktemplate and Utemplate
     // here we allocate enough space for the larger of them to serve both
     //
@@ -1637,20 +1701,20 @@ namespace Tagger {
     int nwords = 0;
     if ( knowntemplateflag ){
       cout << endl << "  Create known words case base" << endl
-	   << "    Timbl options: '" << knownstr << "'" << endl;
-      TimblAPI *UKtree = new TimblAPI( knownstr );
+	   << "    Timbl options: '" << knownstr + commonstr << "'" << endl;
+      TimblAPI *UKtree = new TimblAPI( knownstr + commonstr );
       if ( !UKtree->Valid() )
 	exit(EXIT_FAILURE);
       cout << "    Algorithm = " << to_string(UKtree->Algo())
 	   << endl << endl;
       if ( !piped_input ){
-	ifstream infile( TestFileName.c_str(), ios::in );
+	string inname = TestFilePath + TestFileName;
+	ifstream infile( inname.c_str(), ios::in );
 	if(infile.bad()){
-	  cerr << "Cannot read from " << TestFileName << endl;
+	  cerr << "Cannot read from " << inname << endl;
 	  return 0;
 	}
-	cout << "    Processing data from the file " 
-	     << TestFileName << "...";
+	cout << "    Processing data from the file " << inname << "...";
 	cout.flush();
 	nwords = makedataset( infile, true );
       }
@@ -1677,20 +1741,20 @@ namespace Tagger {
     int nwords = 0;
     if ( unknowntemplateflag ){
       cout << endl << "  Create unknown words case base" << endl
-	   << "    Timbl options: '" << unknownstr << "'" << endl;
-      TimblAPI *UKtree = new TimblAPI( unknownstr );
+	   << "    Timbl options: '" << unknownstr + commonstr << "'" << endl;
+      TimblAPI *UKtree = new TimblAPI( unknownstr + commonstr );
       if ( !UKtree->Valid() )
 	exit(EXIT_FAILURE);
       cout << "    Algorithm = " << to_string(UKtree->Algo()) 
 	   << endl << endl;
       if ( !piped_input ){
-	ifstream infile( TestFileName.c_str(), ios::in );
+	string inname = TestFilePath + TestFileName;
+	ifstream infile( inname.c_str(), ios::in );
 	if(infile.bad()){
-	  cerr << "Cannot read from " << TestFileName << endl;
+	  cerr << "Cannot read from " << inname << endl;
 	  return 0;
 	}
-	cout << "    Processing data from the file "
-	     << TestFileName << "...";
+	cout << "    Processing data from the file " << inname << "...";
 	cout.flush();
 	nwords = makedataset( infile, false );
       }
@@ -1746,8 +1810,13 @@ namespace Tagger {
   void TaggerClass::parse_create_args( TimblOpts& Opts ){
     string value;
     bool mood;
+    if ( Opts.Find( 'V', value, mood ) ||
+	 Opts.Find( "version", value, mood ) ){
+      // we already identified ourself. just bail out.
+      exit(EXIT_SUCCESS);
+    }
     if ( Opts.Find( '%', value, mood ) ){
-      FilterTreshold = stringTo<int>( value );
+      FilterThreshold = stringTo<int>( value );
     }
     if ( Opts.Find( 'd', value, mood ) ){
       dumpflag=true;
@@ -1797,14 +1866,36 @@ namespace Tagger {
     if ( Opts.Find( 's', value, mood ) ){
       // if a settingsfile option has been specified, use that name
       SettingsFileName = value;
+      // extract the absolute path to the settingsfile
+      string::size_type lastSlash = SettingsFileName.rfind('/');
+      if ( lastSlash != string::npos )
+	SettingsFilePath = SettingsFileName.substr( 0, lastSlash+1 );
+      else
+	SettingsFilePath = "";
     }
     if ( Opts.Find( 'E', value, mood ) ){
       TestFileName = value;
+      // extract the absolute path to the testfile
+      string::size_type lastSlash = TestFileName.rfind('/');
+      if ( lastSlash != string::npos ){
+	TestFilePath = TestFileName.substr( 0, lastSlash+1 );
+	TestFileName = TestFileName.substr( lastSlash+1 );
+      }
+      else
+	TestFilePath = "";
       piped_input = false;
       input_kind = ENRICHED; // an enriched tagged test file specified
     }
     if ( Opts.Find( 'T', value, mood ) ){
       TestFileName = value;
+      // extract the absolute path to the testfile
+      string::size_type lastSlash = TestFileName.rfind('/');
+      if ( lastSlash != string::npos ){
+	TestFilePath = TestFileName.substr( 0, lastSlash+1 );
+	TestFileName = TestFileName.substr( lastSlash+1 );
+      }
+      else
+	TestFilePath = "";
       piped_input = false;
       input_kind = TAGGED; // there is a tagged test file specified
     }
@@ -1833,9 +1924,24 @@ namespace Tagger {
       }
       Opts.Delete( 'D' );
     }
+    if ( TestFileName.empty() ){
+      cerr << "Missing required options. see 'mbt -h'" << endl;
+      exit(EXIT_SUCCESS);
+    }
   }
   
   int TaggerClass::CreateTagger( TimblOpts& Opts ){
+    // 
+    // present yourself to the user
+    //
+    cerr << "mbtg " << VERSION << " (c) ILK and CLiPS 1998 - 2011." << endl
+	 << "Memory Based Tagger Generator" << endl
+	 << "Induction of Linguistic Knowledge Research Group,"
+	 << "Tilburg University" << endl
+	 << "CLiPS Computational Linguistics Group, University of Antwerp"
+	 << endl
+	 << "Based on " << Timbl::VersionName() 
+	 << endl << endl;
     TaggerClass tagger;
     tagger.parse_create_args( Opts );
     tagger.set_default_filenames();
